# encoding: utf-8

"""
Initialization module for python-pptx
"""

__version__ = '0.6.0'


import pptx.exc as exceptions
import sys
sys.modules['pptx.exceptions'] = exceptions
del sys

from pptx.api import Presentation  # noqa

from pptx.opc.constants import CONTENT_TYPE as CT
from pptx.opc.package import PartFactory
from pptx.parts.chart import ChartPart
from pptx.parts.coreprops import CorePropertiesPart
from pptx.parts.image import ImagePart
from pptx.parts.presentation import PresentationPart
from pptx.parts.slide import SlideLayoutPart, SlideMasterPart, SlidePart

content_type_to_part_class_map = {
    CT.PML_PRESENTATION_MAIN: PresentationPart,
    CT.PML_PRES_MACRO_MAIN:   PresentationPart,
    CT.PML_TEMPLATE_MAIN:     PresentationPart,
    CT.PML_SLIDESHOW_MAIN:    PresentationPart,
    CT.OPC_CORE_PROPERTIES:   CorePropertiesPart,
    CT.PML_SLIDE:             SlidePart,
    CT.PML_SLIDE_LAYOUT:      SlideLayoutPart,
    CT.PML_SLIDE_MASTER:      SlideMasterPart,
    CT.DML_CHART:             ChartPart,
    CT.BMP:                   ImagePart,
    CT.GIF:                   ImagePart,
    CT.JPEG:                  ImagePart,
    CT.MS_PHOTO:              ImagePart,
    CT.PNG:                   ImagePart,
    CT.TIFF:                  ImagePart,
    CT.X_EMF:                 ImagePart,
    CT.X_WMF:                 ImagePart,
}

PartFactory.part_type_for.update(content_type_to_part_class_map)

del (
    ChartPart, CorePropertiesPart, ImagePart, SlidePart, SlideLayoutPart,
    SlideMasterPart, PresentationPart, CT, PartFactory
)
